@@ -24,20 +24,24 @@
 #'   \item{\code{"rL1"}}{for the summation of absolute relative errors}
 #'   \item{\code{"LB1"}}{for the summation of absolute errors if outside the boundaries}
 #'   \item{\code{"rB1"}}{for the summation of absolute relative errors if outside the boundaries}
-#'   \item{\code{"rbLasso1"}}{for the summation of absolute relative errors if outside the boundaries plus a Lasso penalty based on the distance from the provided weights}
 #'   \item{\code{"L2"}}{for the summation of square errors}
 #'   \item{\code{"aL2"}}{for the asymmetric summation of square errors}
 #'   \item{\code{"rL2"}}{for the summation of square relative errors}
 #'   \item{\code{"LB2"}}{for the summation of square errors if outside the boundaries}
 #'   \item{\code{"rB2"}}{for the summation of square relative errors if outside the boundaries}
-#'   \item{\code{"rbLasso2"}}{for the summation of square relative errors if outside the boundaries plus a Lasso penalty based on the distance from the provided weights}
+#'   \item{\code{"LC"}}{for the summation of the logcosh errors}
+#'   \item{\code{"aLC"}}{for the asymmetric summation of the logcosh errors}
+#'   \item{\code{"rLC"}}{for the summation of the logcosh relative errors}
+#'   \item{\code{"SE"}}{for the summation of the exponential absolute errors}
+#'   \item{\code{"aSE"}}{for the asymmetric summation of the exponential absolute errors}
+#'   \item{\code{"rSE"}}{for the summation of the exponential absolute relative errors}
 #' }
 #' 
-#' A two-column matrix must be provided to \code{tgtBnds} when \code{objective = "LB1"}, \code{objective = "rB1"}, 
-#' \code{objective = "rbLasso1"}, \code{objective = "LB2"}, \code{objective = "rB2"}, and \code{objective = "rbLasso2"}.
+#' A two-column matrix must be provided to \code{tgtBnds} when \code{objective = "aL1"}, \code{objective = "LB1"}, \code{objective = "rB1"},
+#' \code{objective = "aL2"}, \code{objective = "LB2"}, \code{objective = "rB2"}, \code{objective = "aLC"}, and \code{objective = "aSE"}.
 #' 
-#' The argument \code{scale} must be specified with a vector of positive reals number when \code{objective = "rL1"} 
-#' or \code{objective = "rL2"}.
+#' The argument \code{scale} must be specified with a vector of positive real numbers when \code{objective = "rL1"},
+#' \code{objective = "rL2"}, \code{objective = "rLC"}, or \code{objective = "rSE"}.
 #' 
 #' @return 
 #' A vector of integer weights to be the input of the calibration algorithm
@@ -45,24 +49,26 @@
 #' @examples
 #' library(inca)
 #' set.seed(0)
-#' w <- rpois(150, 4)
-#' data <- matrix(rbinom(150000, 1, .3) * rpois(150000, 4), 1000, 150)
+#' w <- rpois(10, 4)
+#' data <- matrix(rbinom(1000, 1, .3) * rpois(1000, 4), 100, 10)
 #' y <- data %*% w
-#' w <- runif(150, 0, 7.5)
+#' w <- runif(10, 0, 7.5)
 #' rw <- roundWeights(w, ~. + 0, y, lower = 1, upper = 7, sparse = TRUE, data = data)
 #' barplot(table(rw), main = "Rounded weigths")
 #'
 #' @export
 
-"roundWeights" <- function(weights, formula, targets, objective = c("L1", "aL1", "rL1", "LB1", "rB1", "rbLasso1", 
-                                                                    "L2", "aL2", "rL2", "LB2", "rB2", "rbLasso2"), 
+"roundWeights" <- function(weights, formula, targets, objective = c("L1", "aL1", "rL1", "LB1", "rB1",
+                                                "L2", "aL2", "rL2", "LB2", "rB2", "LC", "aLC", "rLC",
+                                                "SE", "aSE", "rSE"),
                            tgtBnds = NULL, lower = -Inf, upper = Inf, scale = NULL, sparse = FALSE,
-                           data = environment(formula)) {
+                           data) {
   if (is.null(tgtBnds)) {
-    if (objective[1] %in% c("LB1", "rB1", "rbLasso1", "LB2", "rB2", "rbLasso2"))
-      stop("\"tgtBnds\" must be specified when \"objective\" is either ", paste("\"LB1\",", "\"rB1\",", "\"rbLasso1\",", "\"LB2\",", "\"rB2\",", "or \"rbLasso2\""))
+    if (objective[1] %in% c("LB1", "aL1", "rB1", "LB2", "aL2", "rB2", "aLC", "aSE"))
+      stop("\"tgtBnds\" must be specified when \"objective\" is either ",
+           paste("\"LB1\",", "\"aL1\",", "\"rB1\",", "\"LB2\",", "\"aL2\",", "\"rB2\",", "\"aLC\",", "or \"aSE\""))
     tgtBnds <- cbind(rep_len(-Inf, length(targets)),
-                     rep_len(Inf, length(targets)))
+                     rep_len( Inf, length(targets)))
   }
   else { 
     tgtBnds <- as.matrix(tgtBnds)
@@ -70,22 +76,22 @@
       stop("\"tgtBnds\" must be a data.frame or a matrix object with two columns")
   }
   if(is.null(scale)) {
-    if (objective[1] %in% c("rL1", "rL2"))
-      warning("The argument \"scale\" should be specified when \"objective\" is either ", paste("\"rL1\"", "or \"rL2\"\n"), 
+    if (objective[1] %in% c("rL1", "rL2", "rLC", "rSE"))
+      warning("The argument \"scale\" should be specified when \"objective\" is either ", 
+              paste("\"rL1\",", "\"rL2\",", "\"rLC\",", "or \"rSE\"\n"),
               "In this case, \"scale\" will be set as a vector of values equal to one by default")
     scale <- rep_len(1, length(targets))
   }
-  mylower <- pmax(ceiling(lower), -abs(weights) * Inf)
-  myupper <- pmin(floor(upper), abs(weights) * Inf)
+  mylower <- pmax(ceiling(lower), rep_len(-Inf, length(weights)))
+  myupper <- pmin(floor(upper), rep_len(Inf, length(weights)))
   wts <- adjWeights(weights, lower = mylower, upper = myupper)
-
-  A <- model.matrix(formula, data = as.data.frame(data))
   if(sparse) {
-    A <- Matrix(A, sparse = TRUE)
-    wts <- .Call('sparse_round', PACKAGE = 'inca', A, targets, wts, tgtBnds, scale, objective[1])
+    A <- Matrix::sparse.model.matrix(formula, data = as.data.frame(data))
+    wts <- .Call('sparse_round', PACKAGE = 'inca', A, targets, wts, tgtBnds, scale, objective[1L])
   }
   else {
-    wts <- .Call('dense_round', PACKAGE = 'inca', A, targets, wts, tgtBnds, scale, objective[1])
+    A <- model.matrix(formula, data = as.data.frame(data))
+    wts <- .Call('dense_round', PACKAGE = 'inca', A, targets, wts, tgtBnds, scale, objective[1L])
   }
-  return(wts)
+  return(as.vector(wts))
 }
