@@ -13,8 +13,6 @@
 #define LOSS_LB1 4
 #include "rB1.h"
 #define LOSS_rB1 5
-//#include "Lasso1.h"
-#define LOSS_RBLASSO1 6
 
 // Losses based on L2-norm
 #define IS_L2BASED(X) ((X) > 9 && (X) < 100)
@@ -28,14 +26,28 @@
 #define LOSS_LB2 40
 #include "rB2.h"
 #define LOSS_rB2 50
-//#include "Lasso2.h"
-#define LOSS_RBLASSO2 60
+
+// Ohter losses 
+#define IS_OHTERLOSS(X) ((X) > 99 && (X) < 1000)
+#include "logcosh.h"
+#define LOSS_LOGCOSH 100
+#define LOSS_ALOGCOSH 101
+#define LOSS_RLOGCOSH 102
+#include "sumexp.h"
+#define LOSS_SUMEXP 200
+#define LOSS_ASUMEXP 201
+#define LOSS_RSUMEXP 202
+// #include "quantile.h"
+// #define LOSS_QUANTILE 301
+// #define LOSS_RQUANTILE 302
+// #include "expectile.h"
+// #define LOSS_EXPECTILE 401
+// #define LOSS_REXPECTILE 402
 
 // Tools for losses based on boundaries
 #define IS_LB(X) (((X) == LOSS_LB1) || ((X) == LOSS_LB2))
 #define IS_RB(X) (((X) == LOSS_rB1) || ((X) == LOSS_rB2))
-#define IS_RBLASSO(X) (((X) == LOSS_RBLASSO1) || ((X) == LOSS_RBLASSO2))
-#define OFF_WHEN_BND(X, Y) ((X) || IS_LB((Y)) || IS_RB((Y)) || IS_RBLASSO((Y)))
+#define OFF_WHEN_BND(X, Y) ((X) || IS_LB((Y)) || IS_RB((Y)))
 #endif // LOSSES_HPP
 
 #ifndef SIGN_TOOLS
