@@ -17,16 +17,18 @@ int countOOB(const colvec& L, const colvec& U, const colvec& e) {
 }
 
 #include "Losses/Losses.h"
+#include "Penalties/Penalties.h"
 
-template <typename Type> colvec IntProgCalib(const Type& A, const colvec& y, colvec& w, const colvec& dse,
+template <typename Type> colvec IntProgCalib(const Type& A, const colvec& y, colvec& w, const colvec& ref,
                                              const vec& lower, const vec& upper, const mat& Bnds,
-                                             const colvec& scale, const std::string lossType) {
+                                             const colvec& scale, const std::string lossType,
+                                             const std::string penType, const double lambdaPen) {
     /* Integer Programming Calibration algorithm
      * INPUT:
      *   A, matrix of comodities per each farm
      *   y, vector of targets
      *   w, vector of initial weights
-     *   dse, vector of dse weights as reference vector (used only for LASSO type objective)
+     *   ref, vector of weights as reference vector (used only on the penalty)
      *   lower, vector of lower bounds for Restricted weights
      *   upper, vector of lower bounds for Restricted weights
      *   Bnds, matrix of target-bounds centered for zeros point targets
@@ -37,13 +39,27 @@ template <typename Type> colvec IntProgCalib(const Type& A, const colvec& y, col
      *     * "rL1", summation of absolute relative errors
      *     * "LB1", summation of absolute errors if outside the boundaries
      *     * "rB1", summation of absolute relative errors if outside the boundaries
-     *     * "rbLasso1", summation of absolute relative errors if outside the boundaries + Lasso based on dse weights
      *     * "L2", summation of square errors
      *     * "aL2", asymmetric summation of square errors
      *     * "rL2", summation of square relative errors
      *     * "LB2", summation of square errors if outside the boundaries
      *     * "rB2", summation of square relative errors if outside the boundaries
-     *     * "rbLasso2", summation of square relative errors if outside the boundaries + Lasso based on dse weights
+     *   penType, string indicating which penalty function to include in the objective function:
+     *     * "l0norm", number of adjustments that are different from zero
+     *     * "lasso", summation of the absolute value of the adjustments
+     *     * "ridge", summation of the adjustments squared
+     *     * "raking", using the sum of raking ratios
+     *     * "minentropy", based on the minimum entropy
+     *     * "quadrat", using the Euclidean distance
+     *     * "quadmod", using the modified Euclidean distance
+     *     * "hellinger", using the Hellinger distance
+     *     * "mcp", using the minimax concave penalty
+     *     * "scad", using the smoothly clipped absolute deviations
+     *     * "relasso", using the summation of the absolute value of the relative adjustments 
+     *     * "modrelasso", using the summation of the absolute value of the modified relative adjustments 
+     *     * "rehuber", using the Huber loss on the relative adjustmnets 
+     *     * "modrehuber", using the Huber loss on the modified relative adjustmnets 
+     *   lambdaPen, a double floating point number used to rescale the value returned by the choosen penality function
      *
      * OUTPUT:
      *   w, vector of calibrated integer weights
@@ -60,11 +76,9 @@ template <typename Type> colvec IntProgCalib(const Type& A, const colvec& y, col
     colvec ee(y.size());
     colvec tmpE(y.size());
     double ffnew, ffcc;
-    double minPen = 0.0, totPen = 0.0;
     int oob, oobo;
     colvec s(y.size());
     colvec sp(w.size());
-    colvec pen(w.size()), tpen(w.size());
     umat ord, nord;
     colvec tau, lambda, iscale;
 
@@ -74,38 +88,48 @@ template <typename Type> colvec IntProgCalib(const Type& A, const colvec& y, col
     if(lossType == "rL1") lt = LOSS_rL1;
     if(lossType == "LB1") lt = LOSS_LB1;
     if(lossType == "rB1") lt = LOSS_rB1;
-    if(lossType == "rbLasso1") lt = LOSS_RBLASSO1;
     if(lossType == "L2") lt = LOSS_L2;
     if(lossType == "aL2") lt = LOSS_aL2;
     if(lossType == "rL2") lt = LOSS_rL2;
     if(lossType == "LB2") lt = LOSS_LB2;
     if(lossType == "rB2") lt = LOSS_rB2;
-    if(lossType == "rbLasso2") lt = LOSS_RBLASSO2;
+    if(lossType == "LC") lt = LOSS_LOGCOSH;
+    if(lossType == "aLC") lt = LOSS_ALOGCOSH;
+    if(lossType == "rLC") lt = LOSS_RLOGCOSH;
+    if(lossType == "SE") lt = LOSS_SUMEXP;
+    if(lossType == "aSE") lt = LOSS_ASUMEXP;
+    if(lossType == "rSE") lt = LOSS_RSUMEXP;
 
+    int pn = 0;
+    if(penType == "l0norm") pn = PEN_L0NORM;
+    if(penType == "lasso")  pn = PEN_LASSO;
+    if(penType == "ridge")  pn = PEN_RIDGE;
+    if(penType == "raking") pn = PEN_RACK;
+    if(penType == "minentropy") pn = PEN_MINENT;
+    if(penType == "quadrat") pn = PEN_QUADRAT;
+    if(penType == "quadmod") pn = PEN_QUADMOD;
+    if(penType == "hellinger") pn = PEN_HELL;
+    if(penType == "mcp") pn = PEN_MCP;
+    if(penType == "scad") pn = PEN_SCAD;
+    if(penType == "relasso") pn = PEN_RELASSO;
+    if(penType == "modrelasso") pn = PEN_MODRELASSO;
+    if(penType == "rehuber") pn = PEN_REHUBER;
+    if(penType == "modrehuber") pn = PEN_MODREHUBER;
+
+    /* Processing based on the loss function */
     switch (lt) {
-    case LOSS_aL1: // bounds asymmetry factor
+    case LOSS_aL1: // bounds asymmetry factorLOSS_RLOGCOSH
     case LOSS_aL2:
+    case LOSS_ALOGCOSH:
+    case LOSS_ASUMEXP:
         tau = Bnds.col(1) / (Bnds.col(1) - Bnds.col(0));
         break;
     case LOSS_rL1: // scale factor for relative errors
     case LOSS_rL2:
+    case LOSS_RLOGCOSH:
+    case LOSS_RSUMEXP:
         iscale = 1.0 / scale;
         break;
-    case LOSS_RBLASSO1:
-    case LOSS_RBLASSO2:
-        for (i = 0; (size_t) i < w.size(); i++) {
-            if (w[i] < lower[i]) {
-                w[i] = lower[i];
-            }
-            else {
-                if (w[i] > upper[i]) {
-                    w[i] = upper[i];
-                }
-            }
-        }
-        w = round(w);
-        minPen = sum(abs(dse - w));
-        break;
     default:
         break;
     }
@@ -139,14 +163,6 @@ template <typename Type> colvec IntProgCalib(const Type& A, const colvec& y, col
         ffnew = rB1::ff(Bnds.col(0), Bnds.col(1), ee);
         s = conv_to<colvec>::from((ee > Bnds.col(1)) - (ee < Bnds.col(0)));
         break;
-    case LOSS_RBLASSO1: // relative L1-norm outside the boundaries and zero within + LASSO penalty based on DSE
-        pen = dse - w;
-        tpen = pen;
-        totPen = L1::ff(pen) - minPen;
-        ffnew = rB1::ff(Bnds.col(0), Bnds.col(1), ee) + totPen;
-        s = conv_to<colvec>::from(ee > Bnds.col(1)) - conv_to<colvec>::from(ee < Bnds.col(0));
-        sp = sign(pen);
-        break;
     case LOSS_L2: // L2-norm minimization
         ffnew = L2::ff(ee);
         break;
@@ -163,13 +179,72 @@ template <typename Type> colvec IntProgCalib(const Type& A, const colvec& y, col
     case LOSS_rB2: // relative L2-norm outside the boundaries and zero within
         ffnew = rB2::ff(Bnds.col(0), Bnds.col(1), ee);
         break;
-    case LOSS_RBLASSO2: // relative L2-norm outside the boundaries and zero within + LASSO penalty based on DSE
-        pen = dse - w;
-        tpen = pen;
-        totPen = L1::ff(pen) - minPen;
-        ffnew = rB2::ff(Bnds.col(0), Bnds.col(1), ee) + totPen;
-        s = conv_to<colvec>::from(ee > Bnds.col(1)) - conv_to<colvec>::from(ee < Bnds.col(0));
-        sp = sign(pen);
+    case LOSS_LOGCOSH:
+        ffnew = Logcosh::ff(ee);
+        break;
+    case LOSS_ALOGCOSH:
+        lambda = abs(tau - (ee < 0.0));
+        ffnew = aLogcosh::ff(ee, lambda);
+        break;
+    case LOSS_RLOGCOSH:
+        ffnew = rLogcosh::ff(ee, iscale);
+        break;
+    case LOSS_SUMEXP:
+        ffnew = Sumexp::ff(ee);
+        break;
+    case LOSS_ASUMEXP:
+        lambda = abs(tau - (ee < 0.0));
+        ffnew = aSumexp::ff(ee, lambda);
+        break;
+    case LOSS_RSUMEXP:
+        ffnew = rSumexp::ff(ee, iscale);
+        break;
+    default:
+        break;
+    }
+    /* Computing the current penalty */
+    switch (pn) {
+    case PEN_L0NORM:
+        ffnew += lambdaPen * l0norm::pen(w, ref);
+        break;
+    case PEN_LASSO:
+        ffnew += lambdaPen * lasso::pen(w, ref);
+        break;
+    case PEN_RIDGE:
+        ffnew += lambdaPen * ridge::pen(w, ref);
+        break;
+    case PEN_RACK:
+        ffnew += lambdaPen * raking::pen(w, ref);
+        break;
+    case PEN_MINENT:
+        ffnew += lambdaPen * min_entropy::pen(w, ref);
+        break;
+    case PEN_QUADRAT:
+        ffnew += lambdaPen * quadrat::pen(w, ref);
+        break;
+    case PEN_QUADMOD:
+        ffnew += lambdaPen * quadmod::pen(w, ref);
+        break;
+    case PEN_HELL:
+        ffnew += lambdaPen * Hellinger::pen(w, ref);
+        break;
+    case PEN_MCP:
+        ffnew += mcp::pen(w, ref, lambdaPen);
+        break;
+    case PEN_SCAD:
+        ffnew += scad::pen(w, ref, lambdaPen);
+        break;
+    case PEN_RELASSO:
+        ffnew += lambdaPen * relasso::pen(w, ref);
+        break;
+    case PEN_MODRELASSO:
+        ffnew += lambdaPen* modrelasso::pen(w, ref);
+        break;
+    case PEN_REHUBER:
+        ffnew += lambdaPen * rehuber::pen(w, ref);
+        break;
+    case PEN_MODREHUBER:    
+        ffnew += lambdaPen * modrehuber::pen(w, ref);
         break;
     default:
         break;
@@ -210,12 +285,6 @@ template <typename Type> colvec IntProgCalib(const Type& A, const colvec& y, col
             s = conv_to<colvec>::from(ee > Bnds.col(1)) - conv_to<colvec>::from(ee < Bnds.col(0));
             grad = rB1::ffGrd(A, ee, Bnds.col(0), Bnds.col(1));
             break;
-        case LOSS_RBLASSO1: // relative L1-norm outside the boundaries and zero within + LASSO penalty based on DSE
-            s = conv_to<colvec>::from((ee > Bnds.col(1)) - (ee < Bnds.col(0)));
-            grad = rB1::ffGrd(A, ee, Bnds.col(0), Bnds.col(1));
-            sp = sign(pen);
-            grad -= sp;
-            break;
         case LOSS_L2: // L2-norm minimization
             if (ac == 0) grad = L2::ffGrd(A, ee);
             break;
@@ -232,17 +301,76 @@ template <typename Type> colvec IntProgCalib(const Type& A, const colvec& y, col
         case LOSS_rB2: // relative L2-norm outside the boundaries and zero within
             if (ac == 0) grad = rB2::ffGrd(A, ee, Bnds.col(0), Bnds.col(1));
             break;
-        case LOSS_RBLASSO2: // relative L2-norm outside the boundaries and zero within + LASSO penalty based on DSE
-            if (ac == 0) {
-                grad = rB2::ffGrd(A, ee, Bnds.col(0), Bnds.col(1));
-                sp = sign(pen);
-                grad -= sp;
-            }
+        case LOSS_LOGCOSH:
+            grad = Logcosh::ffGrd(A, ee);
+            break;
+        case LOSS_ALOGCOSH:
+            lambda = abs(tau - (ee < 0.0));
+            grad = aLogcosh::ffGrd(A, ee, lambda);
+            break;
+        case LOSS_RLOGCOSH:
+            grad = rLogcosh::ffGrd(A, ee, iscale);
+            break;
+        case LOSS_SUMEXP:
+            grad = Sumexp::ffGrd(A, ee);
+            break;
+        case LOSS_ASUMEXP:
+            lambda = abs(tau - (ee < 0.0));
+            grad = aSumexp::ffGrd(A, ee, lambda);
+            break;
+        case LOSS_RSUMEXP:
+            grad = rSumexp::ffGrd(A, ee, iscale);
             break;
         default:
             grad.zeros();
             break;
         }
+        switch (pn) {
+        case PEN_L0NORM:
+            grad += lambdaPen * l0norm::penGrd(w, ref);
+            break;
+        case PEN_LASSO:
+            grad += lambdaPen * lasso::penGrd(w, ref);
+            break;
+        case PEN_RIDGE:
+            grad += lambdaPen * ridge::penGrd(w, ref);
+            break;
+        case PEN_RACK:
+            grad += lambdaPen * raking::penGrd(w, ref);
+            break;
+        case PEN_MINENT:
+            grad += lambdaPen * min_entropy::penGrd(w, ref);
+            break;
+        case PEN_QUADRAT:
+            grad += lambdaPen * quadrat::penGrd(w, ref);
+            break;
+        case PEN_QUADMOD:
+            grad += lambdaPen * quadmod::penGrd(w, ref);
+            break;
+        case PEN_HELL:
+            grad += lambdaPen * Hellinger::penGrd(w, ref);
+            break;
+        case PEN_MCP:
+            grad += mcp::penGrd(w, ref, lambdaPen);
+            break;
+        case PEN_SCAD:
+            grad += scad::penGrd(w, ref, lambdaPen);
+            break;
+        case PEN_RELASSO:
+            grad += lambdaPen * relasso::pen(w, ref);
+            break;
+        case PEN_MODRELASSO:
+            grad += lambdaPen* modrelasso::pen(w, ref);
+            break;
+        case PEN_REHUBER:
+            grad += lambdaPen * rehuber::pen(w, ref);
+            break;
+        case PEN_MODREHUBER:    
+            grad += lambdaPen * modrehuber::pen(w, ref);
+            break;
+        default:
+            break;
+        }
         ord = stable_sort_index(abs(grad), "descend");
 
         /* Reset the counter of any change in vector w */
@@ -263,13 +391,11 @@ template <typename Type> colvec IntProgCalib(const Type& A, const colvec& y, col
                 if (cc[i] < (lower[i] + 1.0)) continue; //{Rprintf("%d - w_i < 2\n", j); continue;}
                 cc[i]--;
                 tmpE = ee + A.col(i);
-                tpen[i]++;
             }
             else {
                 if (cc[i] > (upper[i] - 1.0)) continue; //{Rprintf("%d - w_i > 5\n", j); continue;}
                 cc[i]++;
                 tmpE = ee - A.col(i);
-                tpen[i]--;
             }
             /* Counting the number of missed targets (Out Of Bounds),
              * and the new value of the loss function to optimize. */
@@ -291,9 +417,6 @@ template <typename Type> colvec IntProgCalib(const Type& A, const colvec& y, col
             case LOSS_rB1:
                 ffcc = rB1::ff(Bnds.col(0), Bnds.col(1), tmpE);
                 break;
-            case LOSS_RBLASSO1:
-                ffcc = rB1::ff(Bnds.col(0), Bnds.col(1), tmpE) + totPen - fabs(pen[i]) + fabs(tpen[i]);
-                break;
             case LOSS_L2:
                 ffcc = L2::ff(tmpE);
                 break;
@@ -310,12 +433,75 @@ template <typename Type> colvec IntProgCalib(const Type& A, const colvec& y, col
             case LOSS_rB2:
                 ffcc = rB2::ff(Bnds.col(0), Bnds.col(1), tmpE);
                 break;
-            case LOSS_RBLASSO2:
-                ffcc = rB2::ff(Bnds.col(0), Bnds.col(1), tmpE) + totPen - fabs(pen[i]) + fabs(tpen[i]);
+            case LOSS_LOGCOSH:
+                ffcc = Logcosh::ff(tmpE);
+                break;
+            case LOSS_ALOGCOSH:
+                lambda = abs(tau - (tmpE < 0.0));
+                ffcc = aLogcosh::ff(tmpE, lambda);
+                break;
+            case LOSS_RLOGCOSH:
+                ffcc = rLogcosh::ff(tmpE, iscale);
+                break;
+            case LOSS_SUMEXP:
+                ffcc = Sumexp::ff(tmpE);
+                break;
+            case LOSS_ASUMEXP:
+                lambda = abs(tau - (tmpE < 0.0));
+                ffcc = aSumexp::ff(tmpE, lambda);
+                break;
+            case LOSS_RSUMEXP:
+                ffcc = rSumexp::ff(tmpE, iscale);
                 break;
             default:
                 break;
             }
+            switch (pn) {
+                case PEN_L0NORM:
+                    ffcc += lambdaPen * l0norm::pen(w, ref);
+                    break;
+                case PEN_LASSO:
+                    ffcc += lambdaPen * lasso::pen(w, ref);
+                    break;
+                case PEN_RIDGE:
+                    ffcc += lambdaPen * ridge::pen(w, ref);
+                    break;
+                case PEN_RACK:
+                    ffcc += lambdaPen * raking::pen(w, ref);
+                    break;
+                case PEN_MINENT:
+                    ffcc += lambdaPen * min_entropy::pen(w, ref);
+                    break;
+                case PEN_QUADRAT:
+                    ffcc += lambdaPen * quadrat::pen(w, ref);
+                    break;
+                case PEN_QUADMOD:
+                    ffcc += lambdaPen * quadmod::pen(w, ref);
+                    break;
+                case PEN_HELL:
+                    ffcc += lambdaPen * Hellinger::pen(w, ref);
+                    break;
+                case PEN_MCP:
+                    ffcc += mcp::pen(w, ref, lambdaPen);
+                    break;
+                case PEN_SCAD:
+                    ffcc += scad::pen(w, ref, lambdaPen);
+                    break;
+                case PEN_RELASSO:
+                    ffcc += lambdaPen * relasso::pen(w, ref);
+                    break;
+                case PEN_MODRELASSO:
+                    ffcc += lambdaPen* modrelasso::pen(w, ref);
+                    break;
+                case PEN_REHUBER:
+                    ffcc += lambdaPen * rehuber::pen(w, ref);
+                    break;
+                case PEN_MODREHUBER:    
+                    ffcc += lambdaPen * modrehuber::pen(w, ref);
+                    break;
+                default:
+                    break;
+            }
             /* If both the loss function and the number of missed
              * targets are reduced, then the old point is changed,
              * the gradient is updated and the algorithm starts
@@ -352,8 +538,8 @@ template <typename Type> colvec IntProgCalib(const Type& A, const colvec& y, col
                     s = sign(ee);
                     break;
                 case LOSS_aL1:
-                    j = aL1::updategrd(A, s, ee, grad, ord, j);
-                    s = sign(ee);
+                    lambda = abs(tau - (ee < 0.0));
+                    grad = aL1::ffGrd(A, lambda);
                     break;
                 case LOSS_rL1:
                     j = rL1::updategrd(A, iscale, s, ee, grad, ord, j);
@@ -367,21 +553,11 @@ template <typename Type> colvec IntProgCalib(const Type& A, const colvec& y, col
                     j = rB1::updategrd(A, Bnds, s, ee, grad, ord, j);
                     s = conv_to<colvec>::from(ee > Bnds.col(1)) - conv_to<colvec>::from(ee < Bnds.col(0));
                     break;
-                case LOSS_RBLASSO1:
-                    totPen += fabs(tpen[i]) - fabs(pen[i]); //update the total penalty
-                    pen[i] = tpen[i]; // update the partial penalties before absolute value
-                    j = rB1::updategrd(A, Bnds, s, ee, grad, ord, j);
-                    if (sp[i] != d_sign(pen[i])) {
-                        sp[i] = d_sign(pen[i]); //update sign for partial penalties
-                        grad[i] -= sp[i]; //update gradient
-                        j = -1;
-                    }
-                    grad -= sp;
-                    break;
                 case LOSS_L2:
                     grad = L2::ffGrd(A, ee);
                     break;
                 case LOSS_aL2:
+                    lambda = abs(tau - (ee < 0.0));
                     grad = aL2::ffGrd(A, ee, lambda);
                     break;
                 case LOSS_rL2:
@@ -393,19 +569,76 @@ template <typename Type> colvec IntProgCalib(const Type& A, const colvec& y, col
                 case LOSS_rB2:
                     grad = rB2::ffGrd(A, ee, Bnds.col(0), Bnds.col(1));
                     break;
-                case LOSS_RBLASSO2:
-                    totPen += fabs(tpen[i]) - fabs(pen[i]); //update the total penalty
-                    pen[i] = tpen[i]; // update the partial penalties before absolute value
-                    grad = rB2::ffGrd(A, ee, Bnds.col(0), Bnds.col(1));
-                    if (sp[i] != d_sign(pen[i])) {
-                        sp[i] = d_sign(pen[i]); //update sign for partial penalties
-                        grad[i] -= sp[i]; //update gradient
-                    }
+                case LOSS_LOGCOSH:
+                    grad = Logcosh::ffGrd(A, ee);
+                    break;
+                case LOSS_ALOGCOSH:
+                    lambda = abs(tau - (ee < 0.0));
+                    grad = aLogcosh::ffGrd(A, ee, lambda);
+                    break;
+                case LOSS_RLOGCOSH:
+                    grad = rLogcosh::ffGrd(A, ee, iscale);
+                    break;
+                case LOSS_SUMEXP:
+                    grad = Sumexp::ffGrd(A, ee);
+                    break;
+                case LOSS_ASUMEXP:
+                    lambda = abs(tau - (ee < 0.0));
+                    grad = aSumexp::ffGrd(A, ee, lambda);
+                    break;
+                case LOSS_RSUMEXP:
+                    grad = rSumexp::ffGrd(A, ee, iscale);
                     break;
                 default:
                     break;
                 }
-                if (!IS_L1BASED(lt)) { // if the objective function is based on L2-norm (or L0-norm)
+                switch (pn) {
+                case PEN_L0NORM:
+                    grad += lambdaPen * l0norm::penGrd(w, ref);
+                    break;
+                case PEN_LASSO:
+                    grad += lambdaPen * lasso::penGrd(w, ref);
+                    break;
+                case PEN_RIDGE:
+                    grad += lambdaPen * ridge::penGrd(w, ref);
+                    break;
+                case PEN_RACK:
+                    grad += lambdaPen * raking::penGrd(w, ref);
+                    break;
+                case PEN_MINENT:
+                    grad += lambdaPen * min_entropy::penGrd(w, ref);
+                    break;
+                case PEN_QUADRAT:
+                    grad += lambdaPen * quadrat::penGrd(w, ref);
+                    break;
+                case PEN_QUADMOD:
+                    grad += lambdaPen * quadmod::penGrd(w, ref);
+                    break;
+                case PEN_HELL:
+                    grad += lambdaPen * Hellinger::penGrd(w, ref);
+                    break;
+                case PEN_MCP:
+                    grad += mcp::penGrd(w, ref, lambdaPen);
+                    break;
+                case PEN_SCAD:
+                    grad += scad::penGrd(w, ref, lambdaPen);
+                    break;
+                case PEN_RELASSO:
+                    grad += lambdaPen * relasso::pen(w, ref);
+                    break;
+                case PEN_MODRELASSO:
+                    grad += lambdaPen* modrelasso::pen(w, ref);
+                    break;
+                case PEN_REHUBER:
+                    grad += lambdaPen * rehuber::pen(w, ref);
+                    break;
+                case PEN_MODREHUBER:    
+                    grad += lambdaPen * modrehuber::pen(w, ref);
+                    break;
+                default:
+                    break;
+                }
+                if (!IS_L1BASED(lt) || IS_PENALIZED(pn)) { // if the objective function is based on L2-norm (or L0-norm)
                     nord = stable_sort_index(abs(grad), "descend"); //sort the gradient
                     if (any(conv_to<vec>::from(nord != ord))) { // and calculate the position according to the changes
                         j = -1;
